@@ -30,6 +30,9 @@
 #include <assert.h>
 #include "emulator.h"
 
+#include <errno.h>
+#include <string.h>
+
 #include <stdio.h>
 #include <limits.h>
 #include <unistd.h>
@@ -56,7 +59,15 @@ EuclidEmulator::EuclidEmulator():
 EuclidEmulator::~EuclidEmulator(){
 	for(int i=0; i<15; i++) {
 		gsl_spline2d_free(logklogz2pc_spline[i]);
+		gsl_interp_accel_free(logk2pc_acc[i]);
+		gsl_interp_accel_free(logz2pc_acc[i]);
 	}
+	for (int ipar=0; ipar < 8; ipar++){
+		delete [] univ_legendre[ipar];
+	}
+	// Free data stored when file is opened. Size is taken from having previously printed it out.
+	munmap(pc[0], 7698560);
+
 }
 
 /* FUNCTION TO READ IN THE DATA FILE */
@@ -68,7 +79,6 @@ void EuclidEmulator::read_in_ee2_data_file(){
 	double *kptr;
 	int i, ik, iz, idx = 0;
 
-
 	for(iz = 0; iz < nz; iz++){
 		for(ik = 0; ik < nk; ik++){
 			this->Bvec[iz][ik] = 0.0; // initialize the resulting nlc vector
@@ -85,9 +95,11 @@ void EuclidEmulator::read_in_ee2_data_file(){
 	  fp = open("./ee2_bindata.dat", O_RDONLY);
   }
 	if(fp==-1) {
-		cerr << "Unable to open ./ee2_bindata.dat\n";
+		cerr << "Unable to open ee2_bindata.dat\n";
+		cerr << "Error is: " << strerror(errno) << endl;
     exit(1);
 	}
+
 	// Get the size of the file. //
     int status = fstat(fp, & s);
     size = s.st_size;
@@ -121,6 +133,8 @@ void EuclidEmulator::read_in_ee2_data_file(){
 	// Check if all data has been read in properly
 	assert(idx == size/sizeof(double));
 
+	close(fp);
+
 }
 
 /* 2D INTERPOLATION OF PRINCIPAL COMPONENTS */
@@ -137,11 +151,15 @@ void EuclidEmulator::pc_2d_interp(){
     	logklogz2pc_spline[i] = gsl_spline2d_alloc(gsl_interp2d_bicubic, nk, nz);
     	gsl_spline2d_init(logklogz2pc_spline[i], logk, stp, pc[i], nk, nz);
 	}
+	for (int ipar=0; ipar < 8; ipar++){
+		univ_legendre[ipar] = new double[lmax+1];
+	}
 }
 
 /* COMPUTE NLC */
 //void EuclidEmulator::compute_nlc(Cosmology csm, double* redshift, int n_redshift, double* kmodes, int n_kmodes){
-void EuclidEmulator::compute_nlc(Cosmology csm, vector<double> redshift, int n_redshift){
+void EuclidEmulator::compute_nlc(Cosmology* csm, vector<double> redshift, int n_redshift){
+
 	double pc_weight;
 	double basisfunc;
 	double stp_no[n_redshift];
@@ -157,15 +175,15 @@ void EuclidEmulator::compute_nlc(Cosmology csm, vector<double> redshift, int n_r
 					  << "The current redshift z = " << redshift.at(iz) << " is therefore ignored." << std::endl;
 			continue;
 		}
-		stp_no[iz] = csm.compute_step_number(redshift.at(iz));
+		stp_no[iz] = csm->compute_step_number(redshift.at(iz));
         //printf("nStep(%.2f) = %.4f\n", redshift.at(iz), stp_no[iz]);
 	}
 	//printf("Redshifts mapped to nStep\n");
 
 	// Pre-compute all Legendre polynomials up to order lmax
 	for (int ipar=0; ipar < 8; ipar++){
-		univ_legendre[ipar] = new double[lmax+1];
-		gsl_sf_legendre_Pl_array(lmax, csm.cosmo_tf[ipar], univ_legendre[ipar]);
+		//univ_legendre[ipar] = new double[lmax+1];
+		gsl_sf_legendre_Pl_array(lmax, csm->cosmo_tf[ipar], univ_legendre[ipar]);
 		for (int l=0; l<=lmax; l++){
 			univ_legendre[ipar][l] *= sqrt(2.0*l + 1.0); //normalization
 		}
@@ -202,6 +220,7 @@ void EuclidEmulator::compute_nlc(Cosmology csm, vector<double> redshift, int n_r
 			}
 		}
 	}
+
 	//printf("PCA assembled\n");
 }
 
